@@ -520,7 +520,8 @@ formControl(bool namecheck)
     void *e;			/* the new element */
     const char *typedesc;
     int itype = topTag->itype;
-    bool isradio = itype == INP_RADIO;
+    int isradio = itype == INP_RADIO;
+    int isselect = (itype == INP_SELECT) * 2;
 
     if(currentForm) {
 	topTag->controller = currentForm;
@@ -535,20 +536,16 @@ formControl(bool namecheck)
     if(fo) {
 	topTag->jv = e =
 	   domLink("Element", topTag->name, topTag->id, 0, 0,
-	   "elements", fo, isradio);
+	   "elements", fo, isradio | isselect);
     } else {
 	topTag->jv = e =
-	   domLink("Element", topTag->name, topTag->id, 0, 0, 0, jdoc, isradio);
+	   domLink("Element", topTag->name, topTag->id, 0, 0, 0,
+	   jdoc, isradio | isselect);
     }
     get_js_events();
     if(!e)
 	return;
 
-    if(itype == INP_SELECT) {
-	establish_property_array(e, "options");
-	establish_property_number(e, "selectedIndex", -1, false);
-    }
-
     if(itype <= INP_RADIO) {
 	establish_property_string(e, "value", topTag->value, false);
 	if(itype != INP_FILE) {
@@ -557,7 +554,7 @@ formControl(bool namecheck)
 	}			/* not file */
     }
 
-    if(itype == INP_SELECT)
+    if(isselect)
 	typedesc = topTag->multiple ? "select-multiple" : "select-one";
     else
 	typedesc = inp_types[itype];
@@ -651,7 +648,7 @@ jsdw(void)
     if(!cw->dw)
 	return;
     memcpy(cw->dw + 3, "<html>\n", 7);
-    side = sideBuffer(0, cw->dw + 10, cw->fileName, true);
+    side = sideBuffer(0, cw->dw + 10, -1, cw->fileName, true);
     if(side) {
 	printf("A separate window has been created in buffer %d\n", side);
     } else {
@@ -828,7 +825,7 @@ locateOptions(const struct htmlTag *sel, const char *input,
 		    set_property_bool(t->jv, "selected", false);
 	    }
     }
-    /* all options unselected */
+
     s = input;
     while(*s) {
 	e = 0;
@@ -1228,7 +1225,7 @@ encodeTags(char *html)
 	    }
 	    l -= strlen(new + offset);
 	    new[offset] = 0;
-	    j = sideBuffer(0, currentTA->value, 0, false);
+	    j = sideBuffer(0, currentTA->value, -1, 0, false);
 	    if(j) {
 		currentTA->lic = j;
 		sprintf(hnum, "%c%d<buffer %d%c0>",
@@ -1675,8 +1672,6 @@ encodeTags(char *html)
 	    t->controller = currentSel;
 	    t->lic = nopt++;
 	    t->value = htmlAttrVal(topAttrib, "value");
-	    if(t->value == EMPTYSTRING)
-		t->value = 0;
 	    if(htmlAttrPresent(topAttrib, "selected")) {
 		if(currentSel->lic && !currentSel->multiple)
 		    browseError("multiple options are selected");
@@ -1964,6 +1959,9 @@ encodeTags(char *html)
 	foreach(t, htmlStack) {
 	    char *jsrc;
 	    ev = t->jv;
+/* in case the option has disappeared */
+	    if(t->action == TAGACT_OPTION)
+		goto next_onload;
 	    if(!ev)
 		goto next_onload;
 	    if(t->slash)
@@ -2090,7 +2088,7 @@ htmlParse(char *buf, int remote)
 
 void
 findField(const char *line, int ftype, int n,
-   int *total, int *tagno, char **href, void **evp)
+   int *total, int *realtotal, int *tagno, char **href, void **evp)
 {
     const struct htmlTag *t, **list = cw->tags;
     int nt = 0;			/* number of fields total */
@@ -2155,6 +2153,8 @@ findField(const char *line, int ftype, int n,
 	nm = 0;
     if(total)
 	*total = nrt;
+    if(realtotal)
+	*realtotal = nt;
     if(tagno)
 	*tagno = nm;
     if(!ftype && nm) {
@@ -2224,6 +2224,8 @@ findField(const char *line, int ftype, int n,
 	nm = 0;
     if(total)
 	*total = nt;
+    if(realtotal)
+	*realtotal = nt;
     if(href)
 	*href = nmh;
     else
@@ -2231,9 +2233,10 @@ findField(const char *line, int ftype, int n,
 }				/* findField */
 
 void
-findInputField(const char *line, int ftype, int n, int *total, int *tagno)
+findInputField(const char *line, int ftype, int n, int *total, int *realtotal,
+   int *tagno)
 {
-    findField(line, ftype, n, total, tagno, 0, 0);
+    findField(line, ftype, n, total, realtotal, tagno, 0, 0);
 }				/* findInputField */
 
 bool
@@ -2565,7 +2568,7 @@ resetVar(struct htmlTag *t)
     if(itype == INP_TA) {
 	int cx = t->lic;
 	if(cx)
-	    sideBuffer(cx, t->value, 0, false);
+	    sideBuffer(cx, t->value, -1, 0, false);
     } else if(itype != INP_HIDDEN && itype != INP_SELECT)
 	updateFieldInBuffer(t->seqno, w, 0, false);
 
@@ -2832,7 +2835,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 		value = "on";
 	    goto success;
 	}
-	/* checkable */
+
 	if(itype < INP_FILE) {
 /* Even a hidden variable can be adjusted by js.
  * fetchTextVar allows for this possibility.
@@ -2843,7 +2846,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    nzFree(value);
 	    continue;
 	}
-	/* simple text field */
+
 	if(itype == INP_TA) {
 	    int cx = t->lic;
 	    char *cxbuf;
@@ -2885,7 +2888,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    postNameVal(name, 0, fsep, false, boundary, post, l);
 	    continue;
 	}
-	/* textarea */
+
 	if(itype == INP_SELECT) {
 	    char *display = getFieldFromBuffer(t->seqno);
 	    char *s, *e;
@@ -2901,6 +2904,12 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    nzFree(display);
 	    if(!rc)
 		goto fail;	/* this should never happen */
+/* option could have an empty value, usually the null choice,
+ * before you have made a selection. */
+	    if(!*value) {
+		postNameVal(name, value, fsep, false, boundary, post, l);
+		continue;
+	    }
 /* Step through the options */
 	    for(s = value; *s; s = e) {
 		char more;
@@ -2917,7 +2926,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    nzFree(value);
 	    continue;
 	}
-	/* select */
+
 	if(itype == INP_FILE) {	/* the only one left */
 	    value = fetchTextVar(t);
 	    if(!value)
@@ -3214,7 +3223,6 @@ javaSetsTagVar(void *v, const char *val)
 	return;
 /* ok, we found it */
     if(t->itype == INP_HIDDEN || t->itype == INP_RADIO) {
-/* this should never happen */
 	return;
     }
     if(t->itype == INP_TA) {
