@@ -19,7 +19,7 @@
  */
 
 #include "eb.h"
-#include "tcp.h"
+
 /* You need the open ssl library for secure connections. */
 /* Hence the openSSL exception above. */
 #include <openssl/ssl.h>
@@ -426,7 +426,7 @@ httpConnect(const char *from, const char *url)
 {
     int port;			/* usually 80 */
     const char *portloc;
-    long hip;			/* host IP */
+    IP32bit hip;		/* host IP */
     const char *host, *post, *s;
     char *hdr;			/* http header */
     char *u;
@@ -471,12 +471,21 @@ httpConnect(const char *from, const char *url)
     host = getHostURL(url);
     if(!host)
 	errorPrint("@empty host in httpConnect");
-    hip = tcp_name_ip(host);
+    if(proxy_host) {
+	if(secure) {
+	    setError("secure proxy not yet implemented");
+	    return false;
+	}
+	hip = tcp_name_ip(proxy_host);
+    } else {
+	hip = tcp_name_ip(host);
+    }
     if(hip == -1) {
 	setError(intFlag ? opint : "cannot identify %s on the network", host);
 	return false;
     }
-    debugPrint(4, "%s -> %s", host, tcp_ip_dots(hip));
+    debugPrint(4, "%s -> %s",
+       (proxy_host ? proxy_host : host), tcp_ip_dots(hip));
 
 /* See if the protocol is a recognized stream */
     if(stringEqualCI(prot, "http") || stringEqualCI(prot, "https")) {
@@ -539,12 +548,15 @@ httpConnect(const char *from, const char *url)
     newurl = false;
 
     getPortLocURL(url, &portloc, &port);
-    hsock = tcp_connect(hip, port, webTimeout);
+    hsock = tcp_connect(hip, (proxy_host ? proxy_port : port), webTimeout);
     if(hsock < 0) {
 	setError(intFlag ? opint : "cannot connect to %s", host);
 	return false;
     }
-    debugPrint(4, "connected to port %d", port);
+    if(proxy_host)
+	debugPrint(4, "connected to port %d/%d", proxy_port, port);
+    else
+	debugPrint(4, "connected to port %d", port);
     if(secure) {
 	hssl = SSL_new(sslcx);
 	SSL_set_fd(hssl, hsock);
@@ -574,7 +586,19 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	post++;
 
     hdr = initString(&l);
-    stringAndString(&hdr, &l, post ? "POST /" : "GET /");
+    stringAndString(&hdr, &l, post ? "POST " : "GET ");
+    if(proxy_host) {
+	stringAndString(&hdr, &l, prot);
+	stringAndString(&hdr, &l, "://");
+	stringAndString(&hdr, &l, host);
+/* Some proxy servers won't accept :80 after the hostname.  Dunno why. */
+	if(secure || port != 80) {
+	    stringAndChar(&hdr, &l, ':');
+	    stringAndNum(&hdr, &l, port);
+	}
+    }
+    stringAndChar(&hdr, &l, '/');
+
     s = getDataURL(url);
     while(true) {
 	char c, buf[4];
@@ -583,6 +607,8 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	c = *s;
 	if(!c)
 	    break;
+	if(c == '#')
+	    break;
 	if(c == '\\')
 	    c = '/';
 	if(c <= ' ') {
@@ -632,7 +658,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 /* Web caching not yet implemented. */
     stringAndString(&hdr, &l,
        "Pragma: no-cache\r\nCache-Control: no-cache\r\n");
-    stringAndString(&hdr, &l, "Accept-Encoding: gzip, compress, deflate\r\n");
+    stringAndString(&hdr, &l, "Accept-Encoding: gzip, compress\r\n");
     stringAndString(&hdr, &l, "Accept-Language: en\r\n");
     if(u = getAuthString(url)) {
 	stringAndString(&hdr, &l, u);
@@ -807,7 +833,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	    fflush(stdout);
 	    fflush(stdin);
 	    if(!fgets(user, sizeof (user), stdin))
-		exit(0);
+		ebClose(0);
 	    n = strlen(user);
 	    if(n >= sizeof (user) - 1) {
 		printf("user name too long, limit %d characters",
@@ -824,7 +850,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	    fflush(stdout);
 	    fflush(stdin);
 	    if(!fgets(pass, sizeof (pass), stdin))
-		exit(0);
+		ebClose(0);
 	    n = strlen(pass);
 	    if(n >= sizeof (pass) - 1) {
 		printf("user name too long, limit %d characters",
@@ -1058,9 +1084,9 @@ ftpls(char *line)
 	    break;
     if(j == 10 && line[j] == ' ') {	/* long list */
 	int fsize, nlinks;
-	char user[40], group[40];
+	char user[42], group[42];
 	char *q;
-	sscanf(line + j, " %d %s %s %d", &nlinks, user, group, &fsize);
+	sscanf(line + j, " %d %40s %40s %d", &nlinks, user, group, &fsize);
 	q = strchr(line, ':');
 	if(q) {
 	    for(++q; isdigitByte(*q) || *q == ':'; ++q) ;
@@ -1239,10 +1265,10 @@ This only works on the current buffer.
 void
 allIPs(void)
 {
-    static long iplist[5 + 1];
+    static IP32bit iplist[5 + 1];
     char *domlist[8];
     int iptotal = 0, domtotal = 0;
-    long ip;
+    IP32bit ip;
     int ntags, tagno;
     char *href;
     const char *dom;
@@ -1253,9 +1279,9 @@ allIPs(void)
     for(ln = 1; ln <= cw->dol; ++ln) {
 	p = (char *)fetchLine(ln, -1);
 	ftype = 0;		/* input stuff doesn't work */
-	findField(p, ftype, 0, &nf, 0, 0, 0);
+	findField(p, ftype, 0, &nf, 0, 0, 0, 0);
 	for(j = 1; j <= nf; ++j) {
-	    findField(p, ftype, j, &nf, &tagno, &href, 0);
+	    findField(p, ftype, j, &nf, 0, &tagno, &href, 0);
 	    if(!href)
 		continue;
 	    if(dom = getHostURL(href)) {
@@ -1277,7 +1303,7 @@ allIPs(void)
 			ip = tcp_dots_ip(dom);
 		    else
 			ip = tcp_name_ip(dom);
-		    if(ip != -1) {
+		    if(ip != NULL_IP) {
 /* could be a repeat */
 			for(k = 0; k < iptotal; ++k)
 			    if(iplist[k] == ip)
@@ -1301,7 +1327,7 @@ allIPs(void)
     }				/* loop over lines */
 
   done:
-    iplist[iptotal] = -1;
+    iplist[iptotal] = NULL_IP;
     cw->iplist = iplist;
     for(k = 0; k < domtotal; ++k)
 	nzFree(domlist[k]);
