@@ -986,12 +986,13 @@ javaParseExecute(void *this, const char *str, const char *filename, int lineno)
 void *
 domLink(const char *classname,	/* instantiate this class */
    const char *symname, const char *idname, const char *href, const char *href_url, const char *list,	/* next member of this array */
-   void *owner, bool isradio)
+   void *owner, int radiosel)
 {
     JSObject *v = 0, *w, *alist = 0, *master;
     jsval vv, listv;
     jsuint length, attr = PROP_FIXED;
     JSClass *cp;
+    bool dupname = false;
     int i;
 
     if(cw->jsdead)
@@ -1002,29 +1003,82 @@ domLink(const char *classname,	/* instantiate this class */
 	if(stringEqual(cp->name, classname))
 	    break;
 
-    if(isradio) {
+    if(symname) {
 	JSBool found;
 	JS_HasProperty(jcx, owner, symname, &found);
 	if(found) {
-	    JS_GetProperty(jcx, owner, symname, &vv);
-	    v = JSVAL_TO_OBJECT(vv);
+
+/*********************************************************************
+This could be a duplicate name.
+Yes, that really happens.
+Link to the first tag having this name,
+and link the second tag under a fake name, so gc won't throw it away.
+Or - it could be a duplicate name because multiple radio buttons
+all share the same name.
+The first time we create the array, and thereafter we just link
+under that array.
+Or - and this really does happen -
+an input tag could have the name action, colliding with form.action.
+I have no idea what to do here.
+I will assume the tag displaces the action.
+That means javascript cannot change the action of the form,
+which it rarely does anyways.
+When it refers to form.action, that will be the input tag.
+I'll check for that one first.
+Ok???
+Yeah, it makes my head spin too.
+*********************************************************************/
+
+	    if(stringEqual(symname, "action")) {
+		JSObject *ao;	/* action object */
+		JS_GetProperty(jcx, owner, symname, &vv);
+		ao = JSVAL_TO_OBJECT(vv);
+/* actioncrash tells me if we've already had this collision */
+		JS_HasProperty(jcx, ao, "actioncrash", &found);
+		if(!found) {
+		    JS_DeleteProperty(jcx, owner, symname);
+/* gc will clean this up later */
+/* advance, as though this were not found */
+		    goto afterfound;
+		}
+	    }
+
+	    if(radiosel == 1) {
+		JS_GetProperty(jcx, owner, symname, &vv);
+		v = JSVAL_TO_OBJECT(vv);
+	    } else {
+		dupname = true;
+	    }
 	}
     }
 
+  afterfound:
     if(!v) {
-	if(isradio) {
+	if(radiosel) {
 	    v = JS_NewArrayObject(jcx, 0, NULL);
-	    establish_property_string(v, "type", "radio", true);
-	} else
+	    if(radiosel == 1) {
+		establish_property_string(v, "type", "radio", true);
+	    } else {
+/* self-referencing - hope this is ok */
+		establish_property_object(v, "options", v);
+		establish_property_number(v, "selectedIndex", -1, false);
+	    }
+	} else {
 	    v = JS_NewObject(jcx, cp, NULL, owner);
+	}
 	vv = OBJECT_TO_JSVAL(v);
 
-	if(symname) {
+	if(symname && !dupname) {
 	    JS_DefineProperty(jcx, owner, symname, vv, NULL, NULL, attr);
+	    if(stringEqual(symname, "action"))
+		establish_property_bool(v, "actioncrash", true, true);
+
+/* link to document.all */
 	    JS_GetProperty(jcx, jdoc, "all", &listv);
 	    master = JSVAL_TO_OBJECT(listv);
 	    establish_property_object(master, symname, v);
 	} else {
+/* tie this to something, to protect it from gc */
 	    JS_DefineProperty(jcx, owner, fakePropName(), vv,
 	       NULL, NULL, JSPROP_READONLY | JSPROP_PERMANENT);
 	}
@@ -1036,12 +1090,12 @@ domLink(const char *classname,	/* instantiate this class */
 	if(alist) {
 	    JS_GetArrayLength(jcx, alist, &length);
 	    JS_DefineElement(jcx, alist, length, vv, NULL, NULL, attr);
-	    if(symname)
+	    if(symname && !dupname)
 		establish_property_object(alist, symname, v);
 	}			/* list indicated */
     }
 
-    if(isradio) {
+    if(radiosel == 1) {
 /* drop down to the element within the radio array, and return that element */
 /* w becomes the object associated with this radio button */
 /* v is, by assumption, an array */
