@@ -300,6 +300,13 @@ setter_loc_href(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
     uo = obj;
     url_initialize(url, false, true);
     uo_href = cloneString(url);
+    if(uo == jwloc || uo == jdloc) {
+	char *t;
+	unpercentURL(uo_href);
+	t = resolveURL(cw->fileName, uo_href);
+	nzFree(uo_href);
+	uo_href = t;
+    }
     return isWinLoc();
 }				/* setter_loc_href */
 
@@ -793,10 +800,13 @@ get_property_url(void *jv, bool doaction)
 	if(!JSVAL_IS_OBJECT(v)) {
 	  badobj:
 	    JS_ReportError(jcx,
-	       "url object is assigned something that I don't understand; I will not be able to fetch the next web page.");
+	       "url object is assigned something that I don't understand; I may not be able to fetch the next web page.");
 	    return 0;
 	}
 	lo = JSVAL_TO_OBJECT(v);
+	JS_HasProperty(jcx, lo, "actioncrash", &found);
+	if(found)
+	    return 0;
 	if(!JS_InstanceOf(jcx, lo, &url_class, emptyArgs))
 	    goto badobj;
 	JS_GetProperty(jcx, lo, "href", &v);
